"""
This module implements a mickey-mouse UserSource which contains persistent
LoginUsers and a Member mix-in class.

This version works with LoginManager 0.8.5 and ZPatterns 0.3.0.  It
may work with later ones.  The external interfaces are now mostly
defined by LoginManager but the addUser and changeUser stuff is still
local to the PTK and may well be changed.

PersistentUserSource and the Member mix-in are meant to attempt to nail
down the external interface.  This should not be taken as an example of a
proper UserSource or LoginUser!!!
"""

from Products.LoginManager.UserSources import BasicUserSource

from Globals import HTMLFile
import Products
import types

class PersistentUserSource(BasicUserSource):
    """
    A PersistentUserSource is a UserSource who's LoginUsers are
    contained as persistent Zope objects.
    """

    meta_type = "Persistent User Source"
    
    __ac_permissions__ = (
        ('Manage users', ('addUser', )),
        )

    def __init__(self, id, title=''):
        BasicUserSource.__init__.im_func(self, id, title)

    def authenticateUser(self, user, password, request):
        if user.__ <> password:
            return 0
        return 1
    
    def rolesForUser(self, user):
        return user.__roles
        
    def domainsForUser(self, user):
        return user.__domains

    # I might get burned with this name, but it's very temporary

    def addUser(self, userid, password, roles, domains):
        user = self.newItem(userid)
        user.__roles = roles
        user.__domains = domains
        user.__ = password
        return user

class MemberMixin:

    __ac_permissions__ = (
        ('Manage users', ('changeUser',)),
        )

    def setMemberProperties(self, REQUEST):
        for ps in self.PropertySheets():
            ps.manage_changeProperties(REQUEST)

    def getHomeFolder(self):
        """Returns the user's home folder object."""
        return self.getPortal().Members[self.getUserName()]
        
    def getHomeUrl(self):
        """Returns the URL to this user's Member folder."""
        return self.getHomeFolder().absolute_url()
    
    def get_toolbox_actions(self, klass=None):
        """Hmmm, fake this for now."""
        return self.toolbox_actions()

    def toolbox_actions(self):
        """Implements the Toolbox Action Provider interface"""
        return [
            {'name': 'My Stuff',
             'url': self.getHomeUrl()+ '/portal_contents',
             'permissions': [],
             'category': 'user'},
            ]

    def mailPassword(self, manager, REQUEST):
        return manager.mailPasswordTemplate(manager, REQUEST, email=self.email, password=self.__)

    def changeUser(self, password, roles, domains):
        """
        Change me!

        These items (pw, roles, domains) will be moved to a designated
        propertysheet in the final implementation.  This method will
        probably hang around after that as a shortcut.
        """
        self.__ = password
        self.__roles = roles
        self.__domains = domains
    
manage_addPersistentUserSourceForm = HTMLFile('addPersistentUserSource',
                                              globals())

def manage_addPersistentUserSource(self, id, title='', REQUEST=None):
    """Add a PUS"""
    ob = PersistentUserSource(id, title)
    return self.Destination()._installPlugIn(ob,
                                             'Added Persistent User Source',
                                             REQUEST)
