from ExtensionClass import Base

class ToolboxActionProvider(Base):
    """
    
    This class is responsible for reporting the actions an object
    would like to publish in the toolbox.

    """

    def toolbox_actions(self):
        """
        Report possible toolbox actions
            
        Return: a tuple of dictionaries
            Each dictionary should describe a toolbox action.
            The items of each dictionary are:
                'name' -- string, the name of the action to be displayed
                          in the toolbox.
                          
                'url' -- the url of the action, to be called by the toolbox
                         if the action is chosen.
                         
                'permissions' -- a sequence of permissions any of which
                                 is sufficient to perform the action.
                                 
                'category' -- string 'user' | 'object' | 'global' which
                              describes the category of the action, this
                              is used by the toolbox to display the action.
        
        Permission: 'View management screens'
        """

        return ()

    def get_toolbox_actions(self, klass=None):
        """
        
        Walk my class hierarchy and return a concatenation of the
        results of the toolbox_actions methods.

        This method should only return one item per unique 'url'
        value.  The value from the class closest to the object's
        actual class should be used.

        """

        if klass is None:
            klass = self.__class__

        filtered_actions = {}
        
        for base in klass.__bases__:
            actions = self.get_toolbox_actions(base)
            for action in actions:
                filtered_actions[action['url']] = action
        if hasattr(klass, 'toolbox_actions'):
            try:
                actions = klass.toolbox_actions(self)
                for action in actions:
                    filtered_actions[action['url']] = action
            except:
                print self.__class__, klass, actions
                raise

        return tuple(filtered_actions.values())
        


