from PlugIns import PlugIn, PlugInGroup, PlugInContainer
from Providers import Provider
import Acquisition
from ComputedAttribute import ComputedAttribute
from Globals import HTMLFile, default__class_init__

_marker = []

class AttributeProviderGroup(PlugInGroup):
    """
    PlugInGroup that manages the more complex registration needs 
    of AttributeProvider PlugIns/Containers
    """

    def _PlugInsChanged(self, container, plugins=()):

        """Update the registry of attribute names"""

        pg = container.__getattr_registry__ = {}
        ps = container.__setattr_registry__ = {}

        for plugin in plugins:

            readnames, writenames = plugin.namesForRegistration(container)

            for n in readnames:
                if not pg.has_key(n):
                    pg[n]=plugin

            for n in writenames: 
                if not ps.has_key(n):
                    ps[n]=plugin









    def manage_registry(self):
        """ """
        g=self.aq_parent.__getattr_registry__
        s=self.aq_parent.__setattr_registry__
        m={}; m.update(g); m.update(s)

        for k in m.keys():
            v = m[k] = {'rd':[],'wr':[]}
            if g.has_key(k): v['rd']=g[k],
            if s.has_key(k): v['wr']=s[k],

        return m.items()

    manage_below = HTMLFile('www/showAttribProviders', globals())

    __ac_permissions__ = (
        ('View management screens',
            ('manage_registry','manage_below')
        ),
    )


default__class_init__(AttributeProviderGroup)


















class AttributeProvider(Provider):
    """
    AttributeProvider example class.  This attribute provider assumes that
    the object containing the attributes is persistent and can save
    whatever is desired.  All of its methods would need to be redefined
    for providers that implement fixed-schema and/or non-persistence-based
    attributes.
    """
    __plugin_kind__ = 'Attribute Provider'

    def namesForRegistration(self,container):
        """
Return a tuple (readnames,writenames) where each is a sequence of attribute
names which this attribute provider supports reading and writing, respectively.
'*' may be included in either or both sequences, which means the provider will
be called if a more specific provider could not be found for a given attribute.
This is so that attribute providers can potentially create attributes
dynamically based on the requested name.
        """
        return (), self.Attributes

    def _AttributeFor(self,client,name,default=None):
        """
        Since persistent attributes are stored in the object, if we get
        here, then we are trying to get something that doesn't exist!
        """
        return default

    def _SetAttributeFor(self,client,name,value):
        """Set the attribute and return true if successful"""
        client.__dict__[name]=value
        client._p_changed = 1
        return 1

    def _DelAttributeFor(self,client,name):
        """Delete the attribute and return true if successful"""
        del client.__dict__[name]
        client._p_changed = 1
        return 1


    # Class Metadata
    
    meta_type='Persistent Internal Attribute Provider'
    
    Attributes = ('*',)

    _properties=(
        {'id':'title', 'type': 'string', 'mode': 'w'},
        {'id':'Attributes', 'type': 'tokens', 'mode': 'w'},
    )

manage_addAttributeProviderForm = HTMLFile('addAttributeProvider', globals())

def manage_addAttributeProvider(
    self, id, title='', REQUEST=None):
    
    """Add an Attribute Provider"""
    ob = AttributeProvider(id, title)
    return self.Destination()._installPlugIn(ob, "Attribute Provider added", REQUEST)
               





















class AcquiredAttributeProvider(AttributeProvider):

    """Provide attributes from parent APC"""

    def namesForRegistration(self,container):
        return (
            self._handledAttrs(
                container.aq_parent.__getattr_registry__.keys()
            ),
            self._handledAttrs(
                container.aq_parent.__setattr_registry__.keys()
            )
        )
        
    def _handledAttrs(self, names):
    
        a = self.Attributes
        
        if '*' in a:
            return names
            
        if '*' in names:
            return a
        
        return filter(lambda x,a=a: x in a, names)


    def _AttributeFor(self,client,name,default=None):
        return self.aq_parent.aq_parent._AttributeFor(client,name,default)

    def _SetAttributeFor(self,client,name,value):
        return self.aq_parent.aq_parent._SetAttributeFor(client,name,value)

    def _DelAttributeFor(self,client,name):
        return self.aq_parent.aq_parent._SetAttributeFor(client,name)

    meta_type='Acquired Attribute Provider'




manage_addAcquiredAttributeProviderForm = HTMLFile('addAcquiredAttributeProvider', globals())

def manage_addAcquiredAttributeProvider(
    self, id, title='', REQUEST=None):
    
    """Add an Acquired Attribute Provider"""
    ob = AcquiredAttributeProvider(id, title)
    return self.Destination()._installPlugIn(ob, "Acquired Attribute Provider added", REQUEST)





class AttributeProviderContainer:

    AttributeProviders = AttributeProviderGroup(
        'AttributeProviders',
        ['Attribute Provider'],
        title = 'Attribute Providers',
        attr  = '__attributeProviders__'
    )

    __plugin_groups__ = (AttributeProviders,)
    __attributeProviders__ = ()

    __getattr_registry__ = {}
    __setattr_registry__ = {}







    






    def _AttributeFor(self, client, name, default):
    
        self = self.aq_inner    # is this really necessary?
        
        d = self.__getattr_registry__
        pp = d.get(name,_marker)
        if pp is _marker: pp=d.get('*',_marker)

        if pp is not _marker:
            pp = pp.__of__(self)
            return pp._AttributeFor(client,name,default)
            
        return default


    def _SetAttributeFor(self, client, name, value):

        self = self.aq_inner    # is this really necessary?
        
        d = self.__setattr_registry__
        pp = d.get(name,_marker)
        if pp is _marker: pp=d.get('*',_marker)

        if pp is not _marker:
            pp = pp.__of__(self)
            return pp._SetAttributeFor(client,name,value)


    def _DelAttributeFor(self, client, name):

        self = self.aq_inner    # is this really necessary?
        
        d = self.__setattr_registry__
        pp = d.get(name,_marker)
        if pp is _marker: pp=d.get('*',_marker)

        if pp is not _marker:
            pp = pp.__of__(self)
            return pp._DelAttributeFor(client,name)








